/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef CLIPEMBEDDER_H
#define CLIPEMBEDDER_H

#include <onnxruntime_cxx_api.h>

#include "autotokenizer.h"

namespace cv {
class Mat;
}

class ClipEmbedder {
public:
    ClipEmbedder();

    std::vector<float> encodeText(const std::string &text);
    std::vector<float> encodeImage(const std::string &visionFile);

private:
    Ort::Session textSession_;
    Ort::Session visionSession_;
    AutoTokenizer tokenizer_;

    std::vector<float> preprocessVision(const std::string& visionFile) const;
    std::vector<float> imageToFloatVector(const cv::Mat& image) const;
    void normalize(std::vector<float> &vec) const;
    float norm(const std::vector<float> &matrix) const;
    void normalizeTokens(std::vector<int64_t> &tokens, int length) const;
};

#endif // CLIPEMBEDDER_H
