/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "segmenttokenizer.h"

namespace SegmentTokenizer {

std::vector<std::string> fixedSizeChunks(const std::string& text, int chunkSize)
{
    std::vector<std::string> chunks;

    size_t startPos = 0;
    while (startPos < text.size()) {
        size_t endPos = startPos + chunkSize;
        // 确保不要在一个 UTF-8 多字节字符的中间分割
        if (endPos < text.size()) {
            while ((text[endPos] & 0xC0) == 0x80) {
                // 如果 endPos 指向的字节是多字节字符的非起始字节
                // 向前移动，直到找到多字节字符的起始字节
                --endPos;
            }
        }

        size_t length = endPos - startPos;
        chunks.push_back(text.substr(startPos, length));
        startPos += length;
    }

    return chunks;
}

void segmentTokenize(std::vector<std::string> &texts, int maxLength)
{
    for (auto begin = texts.begin(); begin != texts.end();) {
        if (begin->size() < maxLength) {
            ++begin;
            continue;
        }

        std::vector<std::string> chunks = fixedSizeChunks(*begin, maxLength);
        begin = texts.erase(begin);
        begin = texts.insert(begin, chunks.begin(), chunks.end());
        std::advance(begin, chunks.size());
    }
}

}
