#ifndef RUNTIMEERROR_H
#define RUNTIMEERROR_H

#include <memory>
#include <string>

// 运行时相关错误分类
enum class RuntimeErrorCategory {
    INITIALIZATION,
    CONNECTIONCAPABILITY,
    TASK
};

// 运行时相关错误码
enum class RuntimeErrorCode {
    // 初始化相关错误
    FAILEDTOSTARTSERVER,
    FAILEDTOSTARTSERVICE,
    NOMOREMEMORY,

    // 连接相关错误
    NOMORECONNECTION,

    // 任务相关错误
    TASKTIMEOUT
};


class RuntimeError {
public:
    ~RuntimeError() = default;

    static std::unique_ptr<RuntimeError> create(RuntimeErrorCategory category, RuntimeErrorCode code, const std::string& message) {
        return std::unique_ptr<RuntimeError>(new RuntimeError(category, code, message));
    }

    const std::string& getModule() const { return module_; }
    RuntimeErrorCategory getCategory() const { return category_; }
    RuntimeErrorCode getCode() const { return code_; }
    const std::string& getMessage() const { return message_; }

private:
    RuntimeError(RuntimeErrorCategory category, RuntimeErrorCode code, const std::string& message)
        : category_(category), code_(code), message_(message) {}

private:
    const std::string module_ = "Runtime";
    const RuntimeErrorCategory category_;
    const RuntimeErrorCode code_;
    const std::string message_;
};

#endif