/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef DATAMANAGEMENTPROCESSORPROXY_H
#define DATAMANAGEMENTPROCESSORPROXY_H

#include <string>

#include "services/datamanagementprocessorglue.h"

class DataManagementProcessorProxy
{
public:
    DataManagementProcessorProxy();
    ~DataManagementProcessorProxy();

    void setUserData(void *userData) { userData_ = userData; }

    int similaritySearch(const std::string& searchConditions, std::string& searchResult);
    int addTextFiles(const std::string& fileinfos);
    int addImageFiles(const std::string& fileinfos);
    int deleteFiles(const std::string& fileinfos);
    int updateFilesName(const std::string& fileinfos);
    int updateContentData(const std::string& fileinfos);
    int getAllFileinfos(std::string& result);

private:
    void init();
    void destroy();

private:
    const std::string objectPath_ = "/org/openkylin/aisdk/datamanagementprocessor";
    void *userData_ = nullptr;
    AisdkDataManagementProcessor *delegate_ = nullptr;
};

#endif // DATAMANAGEMENTPROCESSORPROXY_H
