#ifndef SERVICES_VISIONPROCESSOR_H
#define SERVICES_VISONPROCESSOR_H

#include <set>
#include <string>
#include <kylin-ai/ai-engine/large-model/visionengine.h>
#include <memory>
#include <mutex>
#include "services/visionprocessorglue.h"
#include "engine/aienginepluginmanager.h"
#include "config/prompttemplate.h"

class VisionProcessor {
public:
    explicit VisionProcessor(
            GDBusConnection &connection,
            ai_engine::AiEnginePluginManager &aiEnginePluginManager);

    ~VisionProcessor();
    
private:
    void exportSkeleton();
    void connectSignal();
    void unexportSkeleton();
    void initEngine(const std::string &engineName, const std::string &appId,
        const std::string &apiKey, const std::string &secretKey);

    friend void onHandleInitEngine(AisdkVisionProcessor *delegate,
       GDBusMethodInvocation *invocation, gchar *engineName, gchar *appId,
       gchar *apiKey, gchar *secretKey, gpointer userData);
    friend bool onHandleSetPromptImageSize(AisdkVisionProcessor *delegate,
                                           GDBusMethodInvocation *invocation,
                                           gint imageWidth,
                                           gint imageHeight,
                                           gpointer userData);
    friend bool onHandleSetPromptImageNumber(AisdkVisionProcessor *delegate,
                                             GDBusMethodInvocation *invocation,
                                             gint imageNumber,
                                             gpointer userData);
    friend bool onHandleSetPromptImageStyle(AisdkVisionProcessor *delegate,
                                            GDBusMethodInvocation *invocation,
                                            gint imageStyle,
                                            gpointer userData);
    friend bool onHandlePromptImage(AisdkVisionProcessor *delegate,
                                    GDBusMethodInvocation *invocation,
                                    const gchar *prompt,
                                    gpointer userData);

private:
    bool isExported_ = false;
    AisdkVisionProcessor *delegate_ = nullptr;

    GDBusConnection &connection_;

    static const std::string objectPath_;

    std::unique_ptr<ai_engine::lm::vision::AbstractVisionEngine> visionEngine_;
    ai_engine::AiEnginePluginManager &aiEnginePluginManager_;
    std::unique_ptr<config::PromptTemplate> promptTemplate_;
    std::mutex mutex_;
    bool stopped { false };
    std::string currentPrompt_;
    int actorId_ { -1 };
    bool actorChanged { false };
};

#endif
